#include <stdio.h>
#include <stdlib.h>
#include <sys/kernel.h>
#include <sys/name.h>
#include <sys/proxy.h>
#include <string.h>
#include <math.h>
#include <i86.h>
#include <unistd.h>

#include "da_cache.h"
#include "address.h"

#include "regnames.h"
#include "panthermsg.h"
#include "msd.h"

#define MAXMSDCHANS 	  8		// Max number of MS channels
#define MAXMSDIONS 		 30		// Max number of ions per channel (ie. num windows)
#define SEPCHARS		":,()"
#define MSDIONFILE		"/home/panther/flight/msd.ions"

int currIonWin = 0;

typedef struct {
	float	ion;			// ion number
	float	dwell;			// dwell time (0 < dwell <= 1) (fraction of a second)
} MSDwindow;

MSDwindow MStab[MAXMSDCHANS][MAXMSDIONS];


// This is identical to the function in msd.tmc
// except MStab is used instead of MStable struct.
int ReadMSDionFileNow ( char *IONFILE ) {
	FILE 	*fp;
	char 	buffer[1000];
	char 	*p;
	char	tmp[1];
	int		chan, inc;
	double	ion, dwell;

	fp = fopen(	IONFILE, "r" );
	if (fp == NULL) {
		fprintf(stderr, "Open failed: Cannont find %s\n", IONFILE);
		return -1;
	} else {
		while ( fgets( buffer, 1000, fp ) != NULL ) {

			// First strip out the MS channel number
			p = strstr( strlwr(buffer), "mschan" );
			strncpy(tmp, p + strlen("MSchan"), 1);
			chan = atoi(tmp);
			if (chan >= MAXMSDCHANS)
				break;
		
			// Next strip out the ions and dwell times
			p =	strtok(buffer, SEPCHARS );
			inc = 0;
			while ( p != NULL ) {

				if ( strstr(strlwr(p), "null") != NULL ) {
					MStab[chan][inc].ion = 0;
					MStab[chan][inc].dwell = 0;
					inc++;
				} else {
					ion = atof(p);
					if (ion >= 1) { 		// found ion
						p = strtok( NULL, SEPCHARS );
						dwell = atof(p);
						if (( dwell <= 1 ) && ( dwell > 0 )) {
							MStab[chan][inc].ion = ion;
							MStab[chan][inc].dwell = dwell;
							//printf ("%d,%d: %f %f\n", chan, inc, ion, dwell);
						} else {
							MStab[chan][inc].ion = 0;
							MStab[chan][inc].dwell = 0;
							fprintf(stderr, "Bad dwell time for channel %d, ion %f\n", chan, ion);
						}

						inc++;
					} else {
						MStab[chan][inc].ion = 0;
						MStab[chan][inc].dwell = 0;
						//printf("null ion: chan = %d, inc = %d\n", chan,inc);
					}

				}

				p = strtok( NULL, SEPCHARS );
			}
		}
	}

	fclose(fp);
	return 0;

}

int SwitchIonWin ( int win ) {
	int 	i;
	char 	msg[300], Rmsg[100], tmp[30];
	pid_t	ms_pid;
	static int once;

	if ( once != 1 ) ReadMSDionFileNow( MSDIONFILE );
	once = 1;

	if ( win == -1 )
		currIonWin++;
	else
		currIonWin = win;

	if ( currIonWin >= MAXMSDIONS )
		currIonWin = MAXMSDIONS-1;
	
	sprintf(msg,"%c", msg_msdDefIons);
	for( i = 0; i<MAXMSDCHANS; i++ ) {
		if ( MStab[i][currIonWin].ion != 0 )
			sprintf(tmp, "%3.1f,%1.2f,", MStab[i][currIonWin].ion, MStab[i][currIonWin].dwell );
		else
			sprintf(tmp, "-99.0,0.02,");
		strcat(msg, tmp);
	}
	
	printf("switch ion win msg = %s, %d\n", msg, strlen(msg));
	if (strlen(msg) <= 1 )
		return 0;

	if ( (ms_pid = qnx_name_locate (0, NAME_MSD, 0, NULL)) == -1 )
		printf("failed to find msd driver!\n");
	else {
		if ( Send(ms_pid, msg, Rmsg, sizeof(msg), sizeof(Rmsg)) == -1 )
			printf("error in the sending ions and dwell times to msd. \n");
	}

	// set da_cache memory variable for collection
	cache_write( IonWinNum_Address, currIonWin );

	return 0;
			
}

void sendMSDmessage ( int msg_msd ) {

	char 	MSDMsg[1],
			Rmsg[3];
	pid_t 	msd_pid;

	MSDMsg[0] = msg_msd;

	msd_pid = qnx_name_locate (0, NAME_MSD, 0, NULL);
	if ( msd_pid != -1 )
		Send(msd_pid, MSDMsg, Rmsg, sizeof(MSDMsg), sizeof(Rmsg));
	else
		printf("send message to msd failed\n");

}

void sendMSDdata ( int msg_msd, float data ) {

	char 	MSDMsg[30],
			Rmsg[3];
	pid_t 	msd_pid;

	sprintf( MSDMsg, "%c%f", msg_msd, data );

	msd_pid = qnx_name_locate (0, NAME_MSD, 0, NULL);
	if ( msd_pid != -1 )
		Send(msd_pid, MSDMsg, Rmsg, sizeof(MSDMsg), sizeof(Rmsg));
	else
		printf("send message to msd failed\n");

}

